# Standard operating procedure - Archiving of trio whole exome samples at Edinburgh Parallel Computing Centre

This SOP applies to the archiving of files generated by the alignment, variant calling, and variant prioritization analysis pipeline for trio whole exome samples at the Edinburgh Parallel Computing Centre (EPCC). Scripts are version controlled on the University of Edinburgh gitlab server gitlab.ecdf.ed.ac.uk/igmmbioinformatics/trio-whole-exome. Request access by e-mail: alison.meynert@igmm.ed.ac.uk.

## User requirements

The user will need an account on the EPCC Ultra system. Contact Donald Scobbie (d.scobbie@eppc.ed.ac.uk) for any issues.

## Software requirements

Htslib 1.10.2 and Samtools 1.10 are installed at /home/u035/project/software/bcbio/tools/bin.

## Data requirements

A copy of the human reference genome hg38 is at /home/u035/project/software/bcbio/genomes/Hsapiens/hg38/seq/hg38.fa.

## Definitions

In this document, N is the total number of samples in the batch, and X is the number of families.

Text in angle brackets, e.g. <batch> indicates variable parameters. A variable parameter such as <family1-X> indicates that there are X instances of the parameter, each with their own unique value.

## Notes

1. In all steps below, when log files are completed and no longer used, move them into the sub-directory ‘logs/complete’. This “clean desk” policy makes it easier to see what current jobs are running and which log files require examination.

2. For all jobs with an array option, if there is only one task to be run, omit the ‘-J’ parameter.

3. The tracking file is maintained on the IGMM datastore at /exports/igmm/datastore/IGMM-VariantAnalysis/trio_whole_exome/Batch_status.xlsx.

## Procedure

1. Log in to the EPCC Ultra system. Set the project id environment variable and other general configuration environment variables, and calculate the number of families in the batch. Change to the logs directory – all jobs are submitted from here.

```
ssh user@ultra.epcc.ed.ac.uk
project_id=<EdGE_project_id>
source /home/u035/project/scripts/trio_whole_exome_config.sh
X=`wc -l $PARAMS_DIR/$project_id.family_ids.txt | awk '{print $1}'`
cd /home/u035/project/trio_whole_exome/analysis/logs
```

2. Compress BAM files to CRAM and compare the two files. The output log files should be empty and the files <sample>.cram, <sample>.cram.crai, and <sample>.cram.flagstat.txt should be present for each sample.

```
qsub -v \
  PROJECT_ID=$project_id,VERSION=$version,CONFIG_SH=/home/u035/project/scripts/trio_whole_exome_config.sh \
  -N trio_whole_exome_cram_compression.$project_id -J 1-${X} \
  /home/u035/project/scripts/submit_trio_wes_cram_compression.sh
```

3. Calculate md5 checksums on the project files, excluding the BAM files. Creates the file md5sum.txt at the root of each family’s output directory. Check the files with the calculated md5sums. They should total 30 lines per sample plus 26 lines per family. The log files should be empty.

```
qsub -v \
  PROJECT_ID=$project_id,VERSION=$version,CONFIG_SH=/home/u035/project/scripts/trio_whole_exome_config.sh \
  -N trio_whole_exome_checksums.$project_id -J 1-${X} \
  /home/u035/project/scripts/submit_trio_wes_checksums.sh
cd ../output
wc -l *$project_id*/md5sum.txt
```

4. Calculate md5 checksums on the prioritization sub-directory(ies) for this project and on the qc files for this project. If there is more than one prioritization sub-directory, use a colon-delimited list.

```
priority_dirs=ddmmyyyy #:ddmmyyyy
qsub -v \
  PROJECT_ID=$project_id,VERSION=$version,PRIORITY_DIRS=$priority_dirs,CONFIG_SH=/home/u035/project/scripts/trio_whole_exome_config.sh \
  -N trio_whole_exome_priority_and_qc_checksums.$project_id \
  /home/u035/project/scripts/submit_trio_wes_priority_and_qc_checksums.sh
```

5. Archive the output to /archive/u035/trio_whole_exome, excluding the BAM files. Confirm md5 checksums on the archived files. There should be no lines ending in ‘FAIL’. If there are any, investigate which file(s) and manually copy these, then re-check the md5 of the archived copy.

```
qsub -v \
  PROJECT_ID=$project_id,VERSION=$version,PRIORITY_DIRS=$priority_dirs,CONFIG_SH=/home/u035/project/scripts/trio_whole_exome_config.sh \
  -N trio_whole_exome_archive_project.$project_id \
  /home/u035/project/scripts/submit_trio_wes_archive_project.sh
grep -c FAIL trio_whole_exome_archive_project.$project_id*
```

6. Clean up the logs directory.

```
cd /home/u035/project/trio_whole_exome/logs 
mv *.project_id.* complete/
```

7. Confirmation of copy to tape. EPCC runs a cron job on Monday mornings at 8am to check the DMF file state for all files in /archive/u035/trio_whole_exome. This produces a text report at /archive/u035/confirmation using the format yyyy-mm-dd. 

Check for ‘DUL’ entries for files in the confirmation report. The grep command below excludes directories, which are generally marked as ‘REG’, blank lines, and total file size lines. If not all entries are DUL, check the non-DUL ones for error states as below. The expectation is that some files may be in the process of migration; in which case, check again in the following week’s report.

```
cd /archive/u035/confirmation
grep -v ^/archive <yyyy-mm-dd> \
  | grep -v total | grep -v ^d | sed '/^$/d' | grep -v DUL
```

In the tracking file, enter the confirmation report date in the ‘Archived’ field for the project.

DMF has several possible states for files. The first three shown below are the most likely to appear:

* REG Regular. The file exists only online, on active disk
* OFL Offline. The file's directory entry remains on disk, but its data blocks are located offline only (on tape).
* DUL Dual-state. Identical copies of the file exist online (on disk) and offline (on tape). The online copy will persist if there is no demand for free space in its filesystem. When free space is needed, the online copy of the file is removed, leaving just the offline copy; the file becomes "offline." If you make any change to a dual-state file, the offline copy becomes out of date and invalid, and the file is once again a "regular" file. 
* MIG Migrating. The file is in process of migrating from disk to tape.
* UNM Unmigrating. The file has been recalled and is in process of moving back from tape to disk.
* NMG Nonmigratable. The file cannot be migrated.
* INV Invalid. DMF cannot determine the file's state. The most likely reason is that it is in a filesystem that does not use DMF