nextflow.enable.dsl = 2


process observed_md5 {
    /* Run md5sum on a file to get its observed checksum */

    input:
    path(downloaded_file)

    output:
    tuple(val("${downloaded_file.getName()}"), stdout)

    script:
    """
    md5sum $downloaded_file | cut -d ' ' -f 1
    """
}

process expected_md5 {
    /*
    Grep out downloaded_file's expected checksum from md5sum_file. Assumes the
    md5sum_file to be in the format '<checksum> path/to/downloaded.file',
    separated by a space
    */

    input:
    path(downloaded_file)
    path(md5sum_file)

    output:
    tuple(val("${downloaded_file.getName()}"), stdout)

    script:
    """
    grep $downloaded_file $md5sum_file | cut -d ' ' -f 1
    """
}

process check_errors {
    /*
    Compare obsered and expected checksums for all given files. If there are
    any, then raise an exception. We have to do this inside a process because
    race conditions occur when trying to execute this directly in a workflow.
    */
    input:
    val(mismatches)
    
    exec:
    if (!mismatches.isEmpty()) {
        throw new PipelineException("MD5 mismatches found: ${mismatches.toString()}")
    }
}


workflow checksums {
    /*
    Take a parsed samplesheet, flatten it and parse into a channel of observed vs.
    expected checksums. Calls check_errors above to raise an exception upon any
    mismatches.
    */

    take:
        ch_samplesheet_info
    
    main:
        ch_fastqs = ch_samplesheet_info.flatMap(
            { indv ->
                [indv[1].r1, indv[1].r2]
            }
        ).flatten()
        .map({file(it)})
        
        ch_md5_files = ch_fastqs.map(
            { fastq -> fastq.getParent().getParent() + '/md5sums.txt' }
        )
        
        ch_obs = observed_md5(ch_fastqs)
        ch_exp = expected_md5(ch_fastqs, ch_md5_files)

        ch_mismatches = ch_obs.concat(ch_exp)
            .map({fastq, md5 -> [fastq, md5.strip()]})
            .groupTuple()
            .filter({it[1][0] != it[1][1]})
            .collect({"${it[0]}: ${it[1][0]} != ${it[1][1]}"})
        
        ch_mismatches.view()
        check_errors(ch_mismatches)
}

/*

merge_fastqs
->
{
    indv1: {
        r1: merged_r1.fastq.gz,
        r2: merged_r2.fastq.gz
    },
    indv2: {
        ...
    }
}


per family:
    prepare_bcbio_config
    ->
    "samplename,description,batch,sex,phenotype,variant_regions"
    "$MERGED_FASTQ,$INDIVIDUAL_ID,$FAMILY_ID,$SEX,$PHENOTYPE,$TWIST_BED_FILE"
    -> 
    family_{family_id}.csv


    bcbio_nextgen.py -w template $BCBIO_TEMPLATE $family{family_id}.csv $READS_DIR/$PROJECT_ID/*_${BARE_FAMILY_ID}_R[12].fastq.gz

*/
